{ This demoprogram shows how to use the function interpreter that's implemented
  in ChroMathsome as well as the other calculus functions.
  The reader may do with this code whatever he/she desires.
  Author: John Merrit (john.merrit@net4all.be)
  Release date: 3 October '96
                                        
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  If you find it useful then I would appreciate any of the following:

  *  Comments, Suggestions, Bug finds etc.
  *  If you improve the interface, please send me a copy of the changes.
  
What I do not want is messages starting with "I am new to Delphi...". 
I've tried to place comments so that these demonstration programs should be readable enough.
Write to me if you have found a problem, not if you have a problem
which can be answered by posting to the comp.lang.delphi.components
newsgroup or any of the mathematics/numerical analysis newsgroups.               }

unit FAnalyst;

interface

uses
    WinTypes, WinProcs, Classes, Graphics, Controls,
    Forms, SysUtils, StdCtrls, Dialogs;

type
  TForm1 = class(TForm)
    FunctionEdit: TEdit;
    UpboundEdit: TEdit;
    LowboundEdit: TEdit;
    StepsEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    EvalButton: TButton;
    ResetButton: TButton;
    RootButton: TButton;
    IntegrateButton: TButton;
    ResultEdit: TEdit;
    Label6: TLabel;
    ValueEdit: TEdit;
    Label7: TLabel;
    Button1: TButton;
    Button2: TButton;
    SeriesButton: TButton;
    CompositeButton: TButton;
    procedure ResetButtonClick(Sender: TObject);
    procedure IntegrateButtonClick(Sender: TObject);
    procedure RootButtonClick(Sender: TObject);
    procedure EvalButtonClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure SeriesButtonClick(Sender: TObject);
    procedure CompositeButtonClick(Sender: TObject);

  private

  function IsDataOk(i: integer): boolean;
  
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses Calculus;
{$R *.DFM}


{ Are the (appropriate) edit boxes filled-in? }
function TForm1.IsDataOk(i: integer): boolean;
var x, y, z: extended;
    j: integer;
begin
    try
    case i of
    1:begin  { Integration }
        y:= StrToFloat(UpboundEdit.Text);
        z:= StrToFloat(LowboundEdit.Text);
        j:= StrToInt(StepsEdit.Text);
      end;
    2:begin  { Root }
         x:= StrToFloat(ValueEdit.Text);
         j:= StrToInt(StepsEdit.Text);
      end;
    3:x:= StrToFloat(ValueEdit.Text);
    end;
    IsDataOk:= true;
    except on EConvertError do IsDataOk:= false;
    end;
end;

{ Reset all edit boxes. }
procedure TForm1.ResetButtonClick(Sender: TObject);
begin
    ResultEdit.Text:= '';
    ValueEdit.Text:= '';
    StepsEdit.Text:= '';
    UpboundEdit.Text:= '';
    LowboundEdit.Text:= '';
    FunctionEdit.Text:= '';
end;

{ Calculate the defined integral of the user-defined function f(x). }
{ Note that, strictly mathematically spoken, the upperbound MUST be greater (or equal to) than the }
{ lowerbound. If you ignore this, a negative value will be returned.   }
procedure TForm1.IntegrateButtonClick(Sender: TObject);
var UpBound, LowBound: extended;
    Steps: integer;
    CalcFunction: string;
    x: extended;
begin
   if IsDataOk(1) = false then ShowMessage('Error in input data!')
   else
   begin
   CalcFunction:= FunctionEdit.Text;
   { procedure PrepFunction adds closing brackets (')') if nescessary. }
   PrepFunction(CalcFunction);
   FunctionEdit.Text:= CalcFunction;
   UpBound:= StrToFloat(UpboundEdit.Text);
   LowBound:= StrToFloat(LowboundEdit.Text);
   Steps:= StrToInt(StepsEdit.Text);
   x:= DefIntegral(CalcFunction,UpBound,LowBound,Steps);
   ResultEdit.Text:= FloatToStr(x);
   end;
end;

{ Find a root for the user-defined function f(x). }
procedure TForm1.RootButtonClick(Sender: TObject);
var CalcFunction: string;
    Invalue: extended;
    Steps: integer;
begin
     if IsDataOk(2) then
     begin
     CalcFunction:= FunctionEdit.Text;
     { procedure PrepFunction adds closing brackets (')') if nescessary. }
     PrepFunction(CalcFunction);
     InValue:= StrToFloat(ValueEdit.Text);
     Steps:= StrToInt(StepsEdit.Text);
     ResultEdit.Text:=  FloatToStr(FunctionRoot(CalcFunction,InValue,Steps));
     end
     else ShowMessage('Error in input data!');
end;

{ Evaluate the user-defined function f(x) at point x0 (in ValuEdit). }
procedure TForm1.EvalButtonClick(Sender: TObject);
var CalcFunction: string;
    x, Value: extended;
begin
   if IsDataOk(3) =  false then ShowMessage('Error in input data!')
   else
   begin
   CalcFunction:= FunctionEdit.Text;
   { procedure PrepFunction adds closing brackets (')') if nescessary. }
   PrepFunction(CalcFunction);
   x:= StrToFloat(ValueEdit.Text);
   Value:= FunctionValue(CalcFunction, x);
   ResultEdit.Text:= FloatToStr(Value);
   end;
end;

{ Display a messagebox with information about the version of ChroMathsome. }
procedure TForm1.Button1Click(Sender: TObject);
begin
     ChroMathsomeVersion;
end;

{ Display a messagebox with information about the time left in the trial period of ChroMathsome. }
procedure TForm1.Button2Click(Sender: TObject);
begin
     HowManyDaysDoIHaveToTestChroMathsome;
end;

{ Calculate the sum of a series.
  This function can be used to test for convergence/divergence of a series.
  (a series is convergent if its sum ...)
  Note that we used the same edit boxes as for integration but now, the numers MUST BE
  longint values!                                                                   }
procedure TForm1.SeriesButtonClick(Sender: TObject);
var UpIndex, LowIndex: LongInt;
    CalcFunction: string;
    x: extended;
begin
   StepsEdit.Text:= '1'; { To avoid an input error in IsDataOk, just ignore this line. }
   if IsDataOk(1) = false then ShowMessage('Error in input data!')
   else
   begin
   CalcFunction:= FunctionEdit.Text;
   { procedure PrepFunction adds closing brackets (')') if nescessary. }
   PrepFunction(CalcFunction);
   FunctionEdit.Text:= CalcFunction;
   UpIndex:= StrToInt(UpboundEdit.Text);
   LowIndex:= StrToInt(LowboundEdit.Text);
   x:= SeriesSum(CalcFunction,UpIndex,LowIndex);
   ResultEdit.Text:= FloatToStr(x);
   end;
end;

{ Create a composite function g(f(x)).
  f(x) defined in Value.Edit;
  g(f) defined in Function.Edit.            }
procedure TForm1.CompositeButtonClick(Sender: TObject);
var CalcFunction, x: string;
begin
     CalcFunction:= FunctionEdit.Text;
     x:= ValueEdit.Text;
     FunctionComposition(CalcFunction, x);
     ResultEdit.Text:= CalcFunction;
end;

end.
